package consumer

import (
	"os"
	"query-service/internal/errorhandler"
	"strconv"
	"time"

	"github.com/streadway/amqp"
	"github.com/thijsheijden/alice"
)

// ConsumeMessageFunc is a function type to be called when a message is consumed
type ConsumeMessageFunc func(*[]byte)

// StartConsuming will start consuming messages
// When a message is received the consumeMessage function will be called
func StartConsuming(consumeMessage ConsumeMessageFunc, exchangeID string, routingKey string) {

	// Create connection config using environment variables
	rabbitUser := os.Getenv("RABBIT_USER")
	rabbitPassword := os.Getenv("RABBIT_PASSWORD")
	rabbitHost := os.Getenv("RABBIT_HOST")
	rabbitPort, err := strconv.Atoi(os.Getenv("RABBIT_PORT"))

	config := alice.CreateConfig(rabbitUser, rabbitPassword, rabbitHost, rabbitPort, true, time.Minute*1, alice.DefaultErrorHandler)

	// Open connection to broker
	conn := alice.Connect(*config)

	// Declare the exchange we want to bind to
	exchange, err := alice.CreateDefaultExchange(exchangeID, alice.Direct)
	if err != nil {
		errorhandler.FailWithError(err, "failed to create exchange")
	}

	// Declare the queue we will consume from
	queue := alice.CreateQueue(exchange, "", true, false, true, false, nil)

	// Create the consumer
	c, err := conn.CreateConsumer(queue, routingKey, alice.DefaultConsumerErrorHandler)
	if err != nil {
		errorhandler.FailWithError(err, "failed to create consumer")
	}

	// Start consuming messages
	c.ConsumeMessages(nil, false, func(msg amqp.Delivery) { consumeMessage(&msg.Body) })
}
